@@ -30,6 +30,14 @@ G_BEGIN_DECLS
 
 libusb_context	*_g_usb_context_get_context	(GUsbContext	*context);
 
+const gchar	*_g_usb_context_lookup_vendor	(GUsbContext	*context,
+						 guint16	 vid,
+						 GError		**error);
+const gchar	*_g_usb_context_lookup_product	(GUsbContext	*context,
+						 guint16	 vid,
+						 guint16	 pid,
+						 GError		**error);
+
 G_END_DECLS
 
 #endif /* __GUSB_CONTEXT_PRIVATE_H__ */
